"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const request_promise_1 = __importDefault(require("request-promise"));
const https_1 = require("https");
const helpers_1 = require("../helpers");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const CFError = require('cf-errors');
const logger = helpers_1.createNewLogger('codefresh:infra:git-providers:bitbucket');
const LIMIT_PER_PAGE = 100;
const MAX_PAGES = 30;
const ApiVersions = {
    // eslint-disable-next-line @typescript-eslint/naming-convention
    V1: 'api/1.0/',
    // eslint-disable-next-line @typescript-eslint/naming-convention
    V2: 'api/2.0/',
    // eslint-disable-next-line @typescript-eslint/naming-convention
    OAUTH: 'site/oauth2/access_token',
};
const _cleanEncodedFilePath = (filepath) => {
    const prefix = './';
    const hasPrefix = filepath.indexOf(prefix) === 0;
    const path = hasPrefix ? filepath.slice(prefix.length) : filepath;
    return encodeURIComponent(path);
};
const _buildErrorString = (response) => {
    let errStr = `${lodash_1.get(response, 'body.error.message')}`;
    if (lodash_1.get(response, 'body.error.id')) {
        errStr = `${errStr} [${lodash_1.get(response, 'body.error.id')}]`;
    }
    return `code: ${response.statusCode}, ${errStr}`;
};
const _toRepo = (rawRepo, issues) => {
    const isGroup = lodash_1.get(rawRepo, 'owner.type') === 'team';
    const httpsClone = rawRepo.links.clone.find((link) => link.name === 'https');
    const sshClone = rawRepo.links.clone.find((link) => link.name === 'ssh');
    return {
        id: rawRepo.uuid,
        provider: 'bitbucket',
        name: rawRepo.name,
        full_name: rawRepo.full_name,
        private: rawRepo.is_private,
        pushed_at: rawRepo.updated_on,
        open_issues: issues ? issues : 0,
        clone_url: lodash_1.get(httpsClone, 'href', ''),
        ssh_url: lodash_1.get(sshClone, 'href', ''),
        owner: {
            login: lodash_1.get(rawRepo, 'owner.display_name', ''),
            avatar_url: lodash_1.get(rawRepo, 'owner.links.avatar.href', ''),
            creator: null,
        },
        org: isGroup ? lodash_1.get(rawRepo, 'owner.username') : null,
        default_branch: lodash_1.get(rawRepo, 'mainbranch.name', ''),
        permissions: {
            admin: true,
        },
        webUrl: lodash_1.get(rawRepo, 'links.html.href', ''),
    };
};
const _toBranch = (rawBranch) => {
    return {
        name: rawBranch.name,
        id: rawBranch.name,
        commit: {
            sha: rawBranch.target.hash,
            commiter_name: lodash_1.get(rawBranch, 'target.author.user.display_name', ''),
            message: lodash_1.get(rawBranch, 'target.message', ''),
            url: lodash_1.get(rawBranch, 'target.links.html.href', ''),
        }
    };
};
class Bitbucket {
    constructor(opts) {
        this.baseUrl = opts.apiURL || 'https://bitbucket.org/';
        if (!this.baseUrl.endsWith('/')) {
            this.baseUrl = `${this.baseUrl}/`;
        }
        this.timeout = opts.timeout || 10000;
        if (opts.type === 'oauth2') {
            // eslint-disable-next-line @typescript-eslint/naming-convention
            this.authenticationHeader = { Authorization: `Bearer ${opts.password}` };
        }
        else {
            // eslint-disable-next-line @typescript-eslint/naming-convention
            this.authenticationHeader = { Authorization: `Basic ${Buffer.from(`${opts.username}:${opts.password}`).toString('base64')}` };
        }
        if (process.env.NODE_TLS_REJECT_UNAUTHORIZED === '0' || opts.insecure) {
            this.agent = new https_1.Agent({ rejectUnauthorized: false });
        }
    }
    getName() {
        return 'bitbucket';
    }
    async performAPICall(opts) {
        const method = opts.method || 'GET';
        const requestHeaders = lodash_1.merge(opts.noAuth ? {} : this.authenticationHeader, opts.headers || {});
        const requestOptions = {
            method,
            url: `${this.baseUrl}${opts.apiVersion}${opts.api}`,
            qs: opts.qs || {},
            headers: requestHeaders,
            json: opts.json,
            timeout: this.timeout,
            form: opts.data,
            resolveWithFullResponse: true,
            agent: this.agent,
            simple: false,
        };
        return request_promise_1.default(requestOptions);
    }
    async paginate(opts, predicate) {
        let start = 1;
        let isLastPage = false;
        const results = [];
        const limit = opts.limit || Number.MAX_SAFE_INTEGER;
        while (!isLastPage && start <= MAX_PAGES && results.length < limit) {
            opts.qs = lodash_1.merge(opts.qs, {
                page: '' + start,
                pagelen: '' + (limit - results.length > LIMIT_PER_PAGE ? LIMIT_PER_PAGE : limit - results.length),
            });
            const res = await this.performAPICall(opts);
            if (res.statusCode >= 400) {
                return res;
            }
            if (predicate) {
                const predicateResult = lodash_1.find(res.body.values, predicate);
                if (predicateResult) {
                    // found the wanted result
                    return {
                        statusCode: 200,
                        body: predicateResult,
                    };
                }
            }
            res.body.values.forEach((result) => { results.push(result); });
            isLastPage = !lodash_1.has(res.body, 'next');
            start += 1;
        }
        return {
            statusCode: 200,
            body: results,
        };
    }
    async fetchRawFile(opts) {
        const res = await this.performAPICall({
            apiVersion: ApiVersions.V2,
            api: `repositories/${opts.owner}/${opts.repo}/src/${opts.ref}/${_cleanEncodedFilePath(opts.path)}`,
        });
        if (res.statusCode >= 400) {
            throw new CFError(`Failed to retrieve file: ${JSON.stringify(opts)}, status code: ${res.statusCode}`);
        }
        return res.body;
    }
    async getBranch(opts) {
        const res = await this.performAPICall({
            apiVersion: ApiVersions.V2,
            api: `repositories/${opts.owner}/${opts.repo}/refs/branches/${opts.branch}`,
            json: true
        });
        if (res.statusCode >= 400) {
            throw new CFError(`Failed to retrieve branch: ${JSON.stringify(opts)}, status code: ${res.statusCode}`);
        }
        return _toBranch(res.body);
    }
    async getRepository(opts) {
        const repoPromise = this.performAPICall({
            apiVersion: ApiVersions.V2,
            api: `repositories/${opts.owner}/${opts.repo}`,
            json: true,
        });
        const issuesPromise = this.performAPICall({
            apiVersion: ApiVersions.V2,
            api: `repositories/${opts.owner}/${opts.repo}/issues`,
            json: true,
        });
        const [repoResponse, issuesResponse] = await Promise.all([repoPromise, issuesPromise]);
        if (repoResponse.statusCode >= 400) {
            throw new CFError(`Failed to get repository: ${JSON.stringify(opts)}, ${_buildErrorString(repoResponse)}`);
        }
        if (issuesResponse.statusCode >= 400) {
            logger.warn(`Failed to get repository issues: ${JSON.stringify(opts)}, ${_buildErrorString(issuesResponse)}`);
            issuesResponse.body.size = 0; // have 0 issues
        }
        return _toRepo(repoResponse.body, issuesResponse.body.size);
    }
    async listBranches(opts) {
        const res = await this.paginate({
            apiVersion: ApiVersions.V2,
            api: `repositories/${opts.owner}/${opts.repo}/refs/branches`,
            json: true,
            page: opts.page,
            limit: opts.limit,
        });
        if (res.statusCode >= 400) {
            throw new CFError(`Failed to get branches list: ${JSON.stringify(opts)}, ${_buildErrorString(res)}`);
        }
        return res.body.map(_toBranch);
    }
    async listRepositoriesForOwner(opt) {
        throw new Error('Method not implemented.');
    }
    async createRepositoryWebhook(opt) {
        throw new Error('Method not implemented.');
    }
    async listWebhooks(opt) {
        throw new Error('Method not implemented.');
    }
    async deleteRepositoryWebhook(opt) {
        throw new Error('Method not implemented.');
    }
    async listRepositoriesWithAffiliation(opt) {
        throw new Error('Method not implemented.');
    }
    async listOrganizations(opt) {
        throw new Error('Method not implemented.');
    }
    async listRepositoriesForOrganization(opt) {
        throw new Error('Method not implemented.');
    }
    async createCommitStatus(opt) {
        throw new Error('Method not implemented.');
    }
}
exports.default = Bitbucket;
//# sourceMappingURL=bitbucket.js.map