"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const rest_1 = require("@octokit/rest");
const plugin_retry_1 = require("@octokit/plugin-retry");
const https_proxy_agent_1 = require("https-proxy-agent");
const lodash_1 = require("lodash");
const https_1 = require("https");
const url_1 = __importDefault(require("url"));
const helpers_1 = require("../helpers");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const CFError = require('cf-errors');
const logger = helpers_1.createNewLogger('codefresh:infra:git-providers:github');
const MAX_PER_PAGE = 100;
const MAX_CONCURRENT_REQUESTS = 4; // requests X pages each time
const MAX_RESULTS = 10000;
const statesMap = {
    pending: 'pending',
    running: 'pending',
    success: 'success',
    failure: 'failure',
    error: 'error',
};
const _getAgent = (insecure) => {
    const options = { keepAlive: true, maxSockets: 10, rejectUnauthorized: !insecure };
    const proxyUrl = process.env.HTTPS_PROXY || process.env.https_proxy;
    let agent;
    if (proxyUrl) {
        const proxy = url_1.default.parse(proxyUrl);
        agent = new https_proxy_agent_1.HttpsProxyAgent({ ...options, ...proxy });
    }
    else {
        agent = new https_1.Agent(options);
    }
    return agent;
};
const _cleanFilePath = (filepath) => {
    const prefix = './';
    const hasPrefix = filepath.indexOf(prefix) === 0;
    const path = hasPrefix ? filepath.slice(prefix.length) : filepath;
    return path;
};
const _toBranch = (rawBranch) => {
    return {
        id: rawBranch.name,
        name: rawBranch.name,
        commit: {
            sha: lodash_1.get(rawBranch, 'commit.sha'),
            commiter_name: lodash_1.get(rawBranch, 'commit.commit.author.name'),
            message: lodash_1.get(rawBranch, 'commit.commit.message'),
            url: lodash_1.get(rawBranch, 'commit.commit.url'),
        }
    };
};
const _toRepo = (rawRepo) => {
    return {
        id: rawRepo.id.toString(),
        provider: 'github',
        name: rawRepo.name,
        full_name: rawRepo.full_name,
        private: rawRepo.private,
        pushed_at: rawRepo.pushed_at,
        open_issues: rawRepo.open_issues_count,
        clone_url: rawRepo.clone_url,
        ssh_url: rawRepo.ssh_url,
        owner: {
            login: lodash_1.get(rawRepo, 'owner.login', ''),
            avatar_url: lodash_1.get(rawRepo, 'owner.avatar_url', ''),
            creator: null,
        },
        org: lodash_1.get(rawRepo, 'organization.login', null),
        default_branch: rawRepo.default_branch,
        permissions: {
            admin: lodash_1.get(rawRepo, 'permissions.admin', null),
        },
        webUrl: lodash_1.get(rawRepo, 'html_url', ''),
    };
};
const _toWebhook = (rawWebhook, owner, repo) => {
    return {
        id: rawWebhook.id,
        name: rawWebhook.name,
        events: rawWebhook.events,
        endpoint: lodash_1.get(rawWebhook, 'config.url', ''),
        repo: `${owner}/${repo}`,
    };
};
class Github {
    constructor(opt) {
        const pathPrefix = (opt.apiPathPrefix && opt.apiPathPrefix !== '/') ? opt.apiPathPrefix : '';
        const agent = _getAgent(process.env.NODE_TLS_REJECT_UNAUTHORIZED === '0' || lodash_1.get(opt, 'insecure', false));
        const GithubClient = rest_1.Octokit.plugin(plugin_retry_1.retry);
        this.githubClient = new GithubClient({
            userAgent: 'Codefresh',
            auth: opt.password,
            request: {
                timeout: opt.timeout || 10000,
                agent,
            },
            timeout: opt.timeout || 10000,
            host: opt.apiHost || 'api.github.com',
            log: logger,
            debug: true,
            baseUrl: `https://${opt.apiHost || 'api.github.com'}${pathPrefix}`
        });
    }
    getName() {
        return 'github';
    }
    async getPaginatedResults(apiFunc, args, limit = MAX_RESULTS, page = 1) {
        const finalResults = [];
        let startingPage = page;
        let shouldStop = false;
        while (!shouldStop) {
            const neededResults = limit - finalResults.length;
            // make up to MAX_PAGES_PER_CYCLE requests in parallel
            const neededPages = Math.min(lodash_1.ceil(neededResults / MAX_PER_PAGE), MAX_CONCURRENT_REQUESTS);
            const pagePromises = lodash_1.range(startingPage, startingPage + neededPages).map((pageNumber) => apiFunc({
                ...args,
                page: pageNumber,
                per_page: MAX_PER_PAGE,
            }));
            // harvest results
            const pagesResults = await Promise.all(pagePromises);
            for (const res of pagesResults) {
                const currentNeededResults = limit - finalResults.length;
                const data = lodash_1.get(res, 'data', []);
                finalResults.push(...data.slice(0, currentNeededResults));
                if (limit === finalResults.length) {
                    shouldStop = true;
                    break; // got all the results we need, no reason to process other requests
                }
                if (lodash_1.isEmpty(data) || lodash_1.size(data) < MAX_PER_PAGE) {
                    shouldStop = true;
                    break; // last page, no reason to process other requests
                }
            }
            // next batch
            startingPage += neededPages;
        }
        return {
            data: finalResults,
            status: 200,
        };
    }
    async fetchRawFile(opt) {
        const { owner, repo, ref, path } = opt;
        const [err, res] = await helpers_1.to(this.githubClient.repos.getContent({ owner, repo, path: _cleanFilePath(path), ref }));
        if (err) {
            throw new CFError({
                message: `Failed to retrieve file: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to retrieve file: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
        const data = res.data;
        const base64Content = data.content.replace(/\n/gi, '');
        return Buffer.from(base64Content, 'base64').toString();
    }
    async getBranch(opt) {
        const [err, res] = await helpers_1.to(this.githubClient.repos.getBranch(opt));
        if (err) {
            throw new CFError({
                message: `Failed to retrieve branch: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to retrieve branch: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
        return _toBranch(res.data);
    }
    async getRepository(opt) {
        const [err, res] = await helpers_1.to(this.githubClient.repos.get(opt));
        if (err) {
            throw new CFError({
                message: `Failed to retrieve repository: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to retrieve repository: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
        return _toRepo(res.data);
    }
    async listBranches(opt) {
        const { owner, repo, limit, page } = opt;
        const [err, res] = await helpers_1.to(this.getPaginatedResults(this.githubClient.repos.listBranches, { owner, repo }, limit, page));
        if (err) {
            throw new CFError({
                message: `Failed to list branches: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to list branches: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
        return res.data.map(_toBranch);
    }
    async listRepositoriesForOwner(opt) {
        const { owner, sort, direction, limit, page } = opt;
        const [err, res] = await helpers_1.to(this.getPaginatedResults(this.githubClient.repos.listForUser, { username: owner, sort, direction }, limit, page));
        if (err) {
            throw new CFError({
                message: `Failed to list repos: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to list repos: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
        return res.data.map(_toRepo);
    }
    async listRepositoriesWithAffiliation(opt) {
        const { limit, page, ...args } = opt;
        const [err, res] = await helpers_1.to(this.getPaginatedResults(this.githubClient.repos.listForAuthenticatedUser, args, limit, page));
        if (err) {
            throw new CFError({
                message: `Failed to list repos: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to list repos: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
        return res.data.map(_toRepo);
    }
    async listRepositoriesForOrganization(opt) {
        const { limit, page, organization, sort, direction } = opt;
        const [err, res] = await helpers_1.to(this.getPaginatedResults(this.githubClient.repos.listForOrg, { org: organization, sort, direction }, limit, page));
        if (err) {
            throw new CFError({
                message: `Failed to list repos: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to list repos: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
        return res.data.map(_toRepo);
    }
    async listWebhooks(opt) {
        const { limit, page, ...args } = opt;
        const [err, res] = await helpers_1.to(this.getPaginatedResults(this.githubClient.repos.listWebhooks, args, limit, page));
        if (err) {
            throw new CFError({
                message: `Failed to list repository webhooks: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to list repository webhooks: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
        return res.data.map((rawWebHook) => _toWebhook(rawWebHook, args.owner, args.repo));
    }
    async createRepositoryWebhook(opt) {
        const { owner, repo, endpoint, secret } = opt;
        const [err, res] = await helpers_1.to(this.githubClient.repos.createWebhook({
            owner,
            repo,
            name: 'web',
            config: {
                url: endpoint,
                content_type: 'json',
                secret,
            },
            events: [
                'push',
                'pull_request',
                'release',
                'issue_comment',
                'repository'
            ],
            active: true
        }));
        if (err) {
            throw new CFError({
                message: `Failed to create repository webhook: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to create repository webhook: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
        return _toWebhook(res.data, owner, repo);
    }
    async deleteRepositoryWebhook(opt) {
        const { owner, repo, hookId } = opt;
        const [err, res] = await helpers_1.to(this.githubClient.repos.deleteWebhook({
            owner,
            repo,
            hook_id: hookId,
        }));
        if (err) {
            throw new CFError({
                message: `Failed to delete repository webhook: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to delete repository webhook: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
    }
    async listOrganizations(opt) {
        const { limit, page } = opt;
        const [err, res] = await helpers_1.to(this.getPaginatedResults(this.githubClient.orgs.listForAuthenticatedUser, {}, limit, page));
        if (err) {
            throw new CFError({
                message: `Failed to list organizations: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to list organizations: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
        return res.data.map((org) => org.login);
    }
    async createCommitStatus(opt) {
        const { owner, repo, sha, targetUrl, state, description, context } = opt;
        const [err, res] = await helpers_1.to(this.githubClient.repos.createCommitStatus({
            owner,
            repo,
            sha,
            target_url: targetUrl,
            state: statesMap[state],
            description,
            context,
        }));
        if (err) {
            throw new CFError({
                message: `Failed to create commit status: ${JSON.stringify(opt)}, status code: ${err.status}`,
                cause: err,
            });
        }
        else if (res.status >= 400) {
            throw new CFError(`Failed to create commit status: ${JSON.stringify(opt)}, status code: ${res.status}`);
        }
    }
}
exports.default = Github;
//# sourceMappingURL=github.js.map